-- drive.e
-- J. Craig Gilbert
-- cgilbert@cennet.mc.peachnet.edu
include wildcard.e
include machine.e

-- local includes
include mach.e

sequence mregs
-----------------------------------------------------------------------------
global function current_drive()
-- returns the letter of the current default drive
   mregs = repeat(0,10)
   mregs[REG_AX] = #1900
   mregs = dos_interrupt(#21,mregs)
   return (and_bits(mregs[REG_AX],#FF) + 'A')
end function
-----------------------------------------------------------------------------
global procedure set_drive(integer letter)
-- sets the current default drive
-- NOTE: even if the drive is not ready, this will set it as the default
-- Will not set the drive if the drive does not exist
   letter = upper(letter)
   if letter < 'A' or letter > 'Z' then
      return
   end if
   mregs = repeat(0,10)
   mregs[REG_AX] = #E00
   mregs[REG_DX] = letter - 'A'
   mregs = dos_interrupt(#21, mregs)
end procedure
-----------------------------------------------------------------------------
global function drive_exist(integer letter)
-- Returns 1 if the drive exists, 0 otherwise
integer current, prev, drive_status

   letter = upper(letter)
   if letter < 'A' or letter > 'Z' then
      return 0
   end if

   prev = current_drive()

   set_drive(letter)
   current = current_drive()
   if current = letter then
      drive_status = 1
   else
      drive_status = 0
   end if

   set_drive(prev)
   
   return drive_status
end function
-----------------------------------------------------------------------------
procedure force_drive_access(integer letter)
-- attempt disk access on the given drive
-- be careful what you use here; dir(), for instance, will cause a
-- problem if the drive is not ready, even if you have your own critical 
-- error handler installed (especially if the handler tells DOS to ignore
-- the error, as mine does)
atom f

   f = open({letter} & ":\\zzwgarbl", "r")
   if f != -1 then
      close(f)
   end if
   
   return
end procedure
-----------------------------------------------------------------------------
integer mCRITICAL_CODE
mCRITICAL_CODE = #FF
global function critical_error_code()
-- Return the code provided by DOS after a critical error
-- Returns #FF if no error occurred
-- Intended to be run after drive_ready() if you want
-- more info on the drive failure; does NOT actually check DOS for the
-- error code, just takes whatever was set by drive_ready()
-- Codes:
--        00  write protect error
--        01  unknown unit
--        02  drive not ready
--        03  unknown command
--        04  data error (bad CRC)
--        05  bad request structure length
--        06  seek error
--        07  unknown media type
--        08  sector not found
--        09  printer out of paper
--        0A  write fault
--        0B  read fault
--        0C  general failure
   return mCRITICAL_CODE
end function
-----------------------------------------------------------------------------
global function drive_ready(integer letter)
-- This function replaces the critical error handler with it's own,
-- attempts to access drive 'letter', then restores the previous
-- critical error handler.
-- Returns 1 if drive ready, 0 if not
-- Does not distinguish between drive not ready because no disk and
-- drive not ready for other reasons (general failure, etc.);
-- call critical_error_code() for more info on the drive failure, if any
sequence new_hand, data_segment
atom handler_addr, status_ptr, errcode_ptr
integer drive_status
   
   drive_status = 0
   
   if not drive_exist(letter) then
      return drive_status
   end if   

   letter = upper(letter)
   
   data_segment = int_to_bytes( get_seg(1) )

   -- create error code location, preloaded w/ #ff (code not used by DOS)
   errcode_ptr = mem_locked(2)   -- high byte is ignored but needs space
   poke(errcode_ptr, #FF)

   -- create error flag location, preloaded w/ 1 for no error
   status_ptr = mem_locked(1)
   poke(status_ptr,1)

   -- construct new handler   
   new_hand = { 
                  -- save stuff
                  #1E,  -- push ds 
                  #53,  -- push ebx
                  -- load data segment
                  #BB} & data_segment & -- mov ebx, data segment value
                { #53,  -- push ebx
                  #1F} &  -- pop ds

                  -- do your thing
                { #FE, #0D } & int_to_bytes(status_ptr) &  -- dec mem
                { #89, #3D } & int_to_bytes(errcode_ptr) & -- mov mem, di

                  -- acknowledge interrupt w/ ignore (zero in al)
                { #32, #C0 } & -- xor eal,eal 

                  -- restore stuff
                { #5B,  -- pop ebx 
                  #1F,  -- pop ds

                  #CF}  -- iretd

   -- set up new handler
   handler_addr = install_handler(new_hand, #24)

   -- access the drive
   force_drive_access(letter)
      
   -- check error status
   drive_status =  peek(status_ptr)
   
   -- get err code
   mCRITICAL_CODE = peek(errcode_ptr)
   
   -- reassign INT 24h to previous critical error handler
   reset_handler(#24)

   free(status_ptr)
   free(errcode_ptr)   
   return drive_status
end function
-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
-----------------------------------------------------------------------------

